# Http.pm: Functions for managing the user web server
#
# $Id: Http.pm,v 1.10.2.1 2001/06/02 03:31:31 pbaltz Exp $
# 
# Copyright 1999 Cobalt Networks http://www.cobaltnet.com
#

package Cobalt::Http;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(http_restart);

use Cobalt::Util;
use Cobalt::Product;

1;

sub http_restart
# Restart the HTTP server
# According to the Apache docs, sending a USR1 signal is the best
# way to signal the server to restart.  This is much faster and
# more graceful than messing with init scripts.
# See http://www.apache.org/docs/stopping.html
#
# Arguments: 1 if we should start up the server when it's not already up
#  (otherwise, if it's not up we'll leave it down).
#            1 (optional) boolean for hard restart, otherwise we gently USR1
#
# Return value: none
#
# Side effects: "The USR1 signal causes the parent process to advise the 
#  children to exit after their current request (or to exit immediately if 
#  they're not serving anything). The parent re-reads its configuration files 
#  and re-opens its log files. As each child dies off the parent replaces it 
#  with a child from the new generation of the configuration, which begins 
#  serving new requests immediately."
{
  my ($bring_it_up, $hard_restart) = @_;

  # set a hard process size limit to work-around the leaking
  # module problem.  Default to 20 MB.  This is the virtual size.
  my $hard_limit = 20000;

  if (! -e "$Subsys/httpd") {
    # The server is not currently running, start it up.
    return unless ($bring_it_up);
    print STDERR "Starting httpd\n" if ($Debug);
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } elsif (! -e $Htpid) {
    # The init script thinks the server is running, but it isn't
    # Clear out the init script's state and start it up.
    return unless ($bring_it_up);
    print STDERR "Restarting httpd\n" if ($Debug);
    system("$Htscript stop >>$Sauce_log 2>&1");
    system("$Htscript start >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } elsif ($bring_it_up == 9) {
    # do a hard restart with the init script
    system("$Htscript restart >>$Sauce_log 2>&1");
    system("$Lcdshowscript >>$Sauce_log 2>&1") if (-x $Lcdshowscript);
  } elsif (!$hard_restart) {
    # The server is running and seems normal
    # Read the PID from the file and send it the signal
    open(PID,$Htpid) || return undef;
    my $pid = <PID>;
    close PID;
    chomp $pid;
    
    # see if the process size has breached the hard limit
    if ($pid)
    {
	`/bin/ps --no-headers -o vsz -p $pid` =~ /(\d+)/;
	if ($1 > $hard_limit)
	{
		# do a hard restart instead of signalling, to stop
		# run away apache growth
		# just be recursive since there is no risk of infinite
		# recursion here
		if ($Debug)
		{
			print STDERR "httpd has breached process hard limit size\n";
			print STDERR "restarting httpd ($pid) with hard restart\n";
		}
		http_restart(9);
	}
	else
	{
    		print STDERR "Signalling httpd ($pid)\n" if ($Debug);
    		kill('USR1',$pid);
	}
    }
    else
    {
	# if we didn't read a pid, do a hard restart, because something is wrong
	http_restart(9);
    }
  } else {
    # hard-restart
    system("$Htscript hard-restart >>$Sauce_log 2>&1");
  }
}

sub get_http_param 
# Report arbitrary conf file arguments
# Arguments: parameter 
#
# Return value: the value of the first configuration file line to begin with the 
#               specified parameter; excluding the parameter itself and one space character
# Side effects: None
{
  my $param=shift;

  open ( CONF, "$Htconf/httpd.conf" ) || 
    return -1;
  while (<CONF>) {
    if (/^${param}\s(.+)/) { 
      close (CONF);
      my $ret = $1;
      $ret =~ s/\s$//g; # strip trailing whitespace
      return $ret;
    }
  }

  close (CONF);
  return -1;
}

sub set_http_param 
# Modify arbitrary conf file arguments
# Arguments: parameter name, parameter value 
#
# Return value: the value of the first configuration file line to begin with the 
#               specified parameter; excluding the parameter itself and one space character
# Side effects: None
{
  my ($param, $spare)=@_;

  my $lock_file = "$Lockdir/httpd.conf";

  # Avoid race
  sleep 1 if (-e $lock_file);
  sleep 1 if (-e $lock_file);
  return MSG_error('spare_set_bad_tmp') if (-e $lock_file);

  open ( CONF, "$Htconf/httpd.conf" ) || 
    return MSG_error("spare_set_bad");
  open ( CONFOUT, "> $lock_file" ) || 
    return MSG_error("spare_set_bad_tmp");
  while (<CONF>) {
    if (/^${param}\s(.+)/) {
      print CONFOUT "$param $spare\n";
    } else {  
      print CONFOUT;
    }
  }
  close (CONF);
  close (CONFOUT);
  rename( $lock_file, "$Htconf/httpd.conf" );
  return MSG_ok("spare_set_good");
}

1;

