#
# $Id: RAID.pm,v 1.33.2.1 2001/06/26 06:27:17 asun Exp $
#
# Software RAID 1 UI interface
# Will DeHaan <will@cobalt.com>
#
# Copyright 2000 Cobalt Networks http://www.cobalt.com/
#

package Cobalt::RAID;

use vars qw(@ISA @EXPORT @EXPORT_OK);

require Exporter;
require SelfLoader;

@ISA    = qw(Exporter SelfLoader);
@EXPORT = qw(raid_get_state raid_get_numdisk
             raid_get_numdisk raid_changelevels 
	    );

my $Debug = 0;
my $ide_smart = '/sbin/ide-smart';

SelfLoader->load_stubs();

1;
__DATA__

sub raid_get_state
# The User Interface (UI) recognizes pairwise HA status by known numeric states.
# This does not make for the most readable app, refer to this table to make
# sense of state ordering:
#
# State #       	State Description                      		UI Descriptor
#  0			Unconfigured/disabled/na			gray
#  1 			Enabled, working great (UU[UU])			green
#  2 			Sync (RAID rebuild), non-fatal			yellow
#  3 			Fatal error, failed disk			red
#
# Integer RAID mode
#  null			RAID unused (state 0)
#  0 			Stripe set
#  1			Mirror (dual disk only)
#  3 			parity disk
#  5			Stripe set w/striped parity
#
# Arguments: none
# Return array of: state code, integer RAID mode (/home only!), 
#                  number of disks used for /home, %completed for sync, 
#                  sync eta in minutes, failed device, 
#                  space separated list (string) of live drive device names for home
{
  my($raid_status, $raid_mode, $raid_disks, $live_disks, $sync_completion, $sync_eta, $failed_device);

  # md1 : active raid1 hdc1[1] hda1[0] 768000 blocks [2/2] [UU]
  # md3 : active raid1 hdc3[1] hda3[0] 205056 blocks [2/2] [UU]
  # md4 : active raid0 hdc4[1] hda4[0] 12750016 blocks 4k chunks
  # md6 : active raid1 hdc6[1] hda6[0] 131456 blocks [2/2] [UU]

  # md4 : active raid5 hdi4[3] hdg4[2] hde4[1] hdk4[0] 33048960 blocks level 5, 32k chunk, algorithm 2 [3/4] [_UUU]
  # md1 : active raid5 hdk1[0] hdi1[3] hdg1[2] hde1[1] 3072000 blocks level 5, 32k chunk, algorithm 2 [4/4] [UUUU]
  # md3 : active raid5 hdk3[0] hdi3[3] hdg3[2] hde3[1] 1536000 blocks level 5, 32k chunk, algorithm 2 [4/4] [UUUU] resync=1% finish=3.9min
  # md6 : active raid5 hdk6[0] hdi6[3] hdg6[2] hde6[1] 394368 blocks level 5, 32k chunk, algorithm 2 [4/4] [UUUU]

  my(%blocks, $blocks_completed, $total_blocks, $raiddev, $home_base);
  open(RAID, "/proc/mdstat") || return (6, '[ha.str.noraid]');
  while(<RAID>) {

    $raiddev = $1 if (/^md(\d)/);

    if (/ (\d+) blocks/) {
      $blocks{$raiddev} = $1;
      $total_blocks += $blocks{$raiddev};
    }

    # only profile /home as we vary RAID modes by partition
    if ($raiddev == 4) {

      $home_base = $_; # remember for failed device analysis

      if (/active raid([\d]+)/) { $raid_mode = $1; }

      # This counts named devices, which should be the same as the [x/y] report
      my($mdstat_home) = $_; # preserve $_ for destructive count
      $raid_disks = ($mdstat_home =~ s/\s([hs]d[a-z]\d.\d.)//g);

      if (/ \[(\d+)\/(\d+)\] /) {
        ($raid_disks, $live_disks) = ($1, $2);
      }
    }

    # failed disk detection and identification; non-RAID 0
    my($mdstat_generic) = $_;
    if (/\/md\d/ && /\[[U\_\-]+\]$/) {

      while ($mdstat_generic =~ /\s[hs]d[a-z]\d.\d./) {
        $mdstat_generic =~ s/\s([hs]d[a-z]\d.\d.)//;
        my($device) = $1;

        if ($mdstat_generic =~ s/\s\[([U\_\-]+)([^\]]*\])$/$2/) {
	  if ($1 eq 'U') {
	    # push @healthy_disks, $device;
          } else {
            $failed_device = $device;
	  }
        }

      }
    }

    # report sync completion
    if (/(resync|recovery)=\s*([\d\.]+)\%.+finish=([\d\.]+)/) {
      my($pct, $eta) = ($2 * 0.01, $3);
      my($fin_blocks) = int($pct * $blocks{$raiddev});
      $blocks_completed += $fin_blocks;
      # $synching = $raiddev; # Flag a partition in recovery

      # Calculate ETA first as time-per-block
      $sync_eta = $eta / ($blocks{$raiddev} - $fin_blocks) if ($blocks{$raiddev} - $fin_blocks);

      $raid_status = 2; # resync
 
    } elsif ((/\[U+\]/) && (!/DELAYED/)) {
      $blocks_completed += $blocks{$raiddev};
    }

    # track failures hdi6[3](F)
    if (/(hd[a-z])\d\[\d\]\(F\)/) {
      $failed_device = $1;
      $raid_status = 3;
    }
  }
  close(RAID);

  # convert eta from TpB to minutes
  $sync_completion = 0;
  if ($sync_eta) {
    $sync_completion = int( 100 * ($blocks_completed / $total_blocks)) if ($total_blocks);
    $sync_eta = 0.1 * int( 10 * $sync_eta * ($total_blocks - $blocks_completed));
  }

  if ($raid_mode !~ /^\d+$/) { # undefined/invalid
    $raid_status = 0;
  } else {
    $raid_status ||= 1;
  }

  # the disk count "[4/4]" has no context in RAID mode 0
  if ($raid_mode > 0) { 
    
    # report absent drives when mdstat does not report failure (F)
    if (($raid_status != 2) && ($live_disks < $raid_disks)) { 
      $raid_status = 3;
    }

    # determine (in an unsophisticated fashion) failed devices post-reboot 
    if (($raid_status == 3) && (!$failed_device)) {

      if ($home_base =~ /hd[ac]/) { 
        # RaQ 3/4 platform, or rulerless 5
	$failed_device = 'hda';
        $failed_device = 'hdc' if ($home_base =~ /hda/);
      } else {
        # Rulered RaQ 5 platform.  From left to right: hde hdg hdi hdk
        $failed_device = 'hde';
        $failed_device = 'hdg' if ($home_base !~ /hdg/);
        $failed_device = 'hdi' if ($home_base !~ /hdi/);
        $failed_device = 'hdk' if ($home_base !~ /hdk/);
      }

    }
  }

  # finally, see if there are any disk problems. this is massively hokey
  # and can potentially confuse the user.
  my $disk;
  foreach $disk ('hda', 'hdc', 'hde', 'hdg', 'hdi', 'hdk') {
	my $smart_ok = system("$ide_smart /dev/$disk 2> /dev/null | grep -q Failed");
	unless ($smart_ok) {
		$failed_device = $disk;
		$raid_status = 3;
		last;
	}	
  }

  # my $healthy_concat = join(' ', @healthy_disks);
  return ($raid_status, $raid_mode, $raid_disks, $sync_completion, $sync_eta, $failed_device, $home_base);
}


sub raid_get_numdisk 
# See how many disks we have in /dev/md1; raid_get_state() reports for md4
# 
# Invoked in the setup wizard for /home setup
# Arguments: none
# Return value: 
{

  open(RAID, "/proc/mdstat") || return (6, '[ha.str.noraid]');
  # We use md1 because that will always be setup to the "reliable" RAID mode 
  while(<RAID>) {
    if (/^md1.*raid0.*/) {
      return 1;
    } 
    if (/^md1.*\[[0-9]\/([0-9])\].*\[.*\]/) {
        return $1;
    }
  } 

  return 0; 
}

sub raid_changelevels {

  my $level=shift;

  system("sh /usr/local/sbin/make_raid.sh $level");

}
