#!/usr/bin/perl

# useful functions... nuff said!
#
# written by Duncan Laurie <duncan@cobaltnet.com>
#

package Cobalt::Util;

use POSIX;
use Locale::gettext;
use Cobalt::Product;
use Fcntl qw( F_WRLCK F_RDLCK F_UNLCK );
use FileHandle;
use Jcode;

require Exporter;
require SelfLoader;
@ISA = qw(Exporter SelfLoader);
@EXPORT = qw(FILE_lock_r FILE_lock_w FILE_lock_rw
	     FILE_unlock FILE_unlock_rw
	     MSG_error MSG_abort MSG_ok MSG_get MSG_get_ui
             parse_cgipost);

@EXPORT_OK = qw( read_status update_status untar_name );

use vars qw($DEBUG);
$DEBUG=0;

SelfLoader->load_stubs();

1;

__DATA__

# Cobalt::Util::MSG_get_ui
#
# generate an message for the UI
# using predetermined locale
#
# arguments are substituted for $arg in the message string
#
sub MSG_get_ui
{
    my $msgkey = shift;
    my @vars = @_;

    my $msg = Locale::gettext::dgettext("ui", $msgkey);
    foreach (@vars) {
	$msg =~ s/\$[^\\\ \.\"\']+/$_/;
    }
    return $msg;
}

# Cobalt::Util::MSG_get
#
# generate an message
# using predetermined locale
#
# arguments are substituted for $arg in the message string
#
sub MSG_get
{
    my $msgkey = shift;
    my @vars = @_;
    my $msg = Locale::gettext::dgettext("special-sauce", $msgkey);
    foreach (@vars) {
	$msg =~ s/\$[^\\\ \.\"\']+/$_/;
    }
    return $msg;
}

# Cobalt::Util::MSG_error
#
# generate an error message prefixed with "4999" from gettext
# using predetermined locale
#
# arguments are substituted for $arg in the message string
#
sub MSG_error
{
    my $msgkey = shift;
    my @vars = @_;
    my $msg = Locale::gettext::dgettext("special-sauce", $msgkey);
    foreach (@vars) {
	$msg =~ s/\$[^\\\ \.\"\']+/$_/;
    }
    $msg = '4999 '.$msg;
    return $msg;
}

# Cobalt::Util::MSG_abort
#
# generate an error message prefixed with "3999" from gettext
# using predetermined locale
#
# arguments are substituted for $arg in the message string
#
sub MSG_abort
{
    my $msgkey = shift;
    my @vars = @_;
    my $msg = Locale::gettext::dgettext("special-sauce", $msgkey);
    foreach (@vars) {
	$msg =~ s/\$[^\\\ \.\"\']+/$_/;
    }
    $msg = '3999 '.$msg;
    return $msg;
}

# Cobalt::Util::MSG_ok
#
# generate a message from the provided key using pre-defined locale
# prefix this message with "2999" to show that it is a good message
#
# arguments are substituted for $arg in the message string
#
sub MSG_ok
{
    my $msgkey = shift;
    my @vars = @_;
    my $msg = Locale::gettext::dgettext("special-sauce", $msgkey);
    foreach (@vars) {
	$msg =~ s/\$[^\\\ \.\"\']+/$_/;
    }
    $msg = '2999 '.$msg;
    return $msg;
}

# Cobalt::Util::FILE_lock_rw
#
# inputs:  filein = name of file to lock (& full path)
# return:  two filehandles, the first for the input and the second for output
#
# creates a lockfile in $Lockdir (/etc/cobalt/locks)
# for writing, while opening the $file for reading
# 
sub FILE_lock_rw
{
    my $filein = shift;
    local *FHIN = shift;
    local *FHOUT = shift;
    my $LOCK_EX = 2;
    my $Lockdir = "/etc/locks";

    # extract filename from fully-qualified filename
    my @fary = split("/", $filein);
    my $ret = pop(@fary);
    my $fileout = "$Lockdir/".pop(@fary)."_$ret";

    # we want to have exclusive locking on the input file
    print STDERR "FILE_lock_rw:  opening $filein... " if ($DEBUG);
    unless(open(FHIN, "< $filein")) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    print STDERR "FILE_lock_rw:  locking $filein... " if ($DEBUG);
    unless(flock(FHIN,$LOCK_EX)) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    print STDERR "FILE_lock_rw:  opening $fileout... " if ($DEBUG);
    unless(open(FHOUT, "> $fileout")) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    print STDERR "FILE_lock_rw:  locking $fileout... " if ($DEBUG);
    unless(flock(FHOUT,$LOCK_EX)) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    return(1);
}

#
# Cobalt::Util::FILE_lock_w
#
# inputs:  file = name of file to lock (& full path)
# return:  filehandle
#
# opens the $file for reading
#
sub FILE_lock_w
{
    my $file = shift;
    local *FH = shift;
    my $LOCK_EX = 2;

    # open the file
    print STDERR "FILE_lock_w:  opening $file... " if ($DEBUG);
    unless(open(FH, "> $file")) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    # lock the file
    print STDERR "FILE_lock_w:  locking $file... " if ($DEBUG);
    unless(flock(FH,$LOCK_EX)) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    return(1);
}

#
# Cobalt::Util::FILE_lock_r
#
# inputs:  file = name of file to lock (& full path)
# return:  filehandle
#
# opens the $file for reading
#
sub FILE_lock_r
{
    my $file = shift;
    local *FH = shift;
    my $LOCK_EX = 2;

    # open the file
    print STDERR "FILE_lock_r:  opening $file... " if ($DEBUG);
    unless(open(*FH,"< $file")) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    # lock the file
    print STDERR "FILE_lock_r:  locking $file... " if ($DEBUG);
    unless(flock(FH,$LOCK_EX)) {
	print STDERR "error!\n" if ($DEBUG);
	return;
    }
    print STDERR "done\n" if ($DEBUG);

    return(1);
}

#
# Cobalt::Util::FILE_unlock
#
# inputs:  array of filehandles to unlock and close
# return:  1 if success
#
sub FILE_unlock
{
    my (@fhs) = @_;
    my ($fh);
    my $LOCK_UN = 8;

    foreach $fh (@fhs) {
	print STDERR "FILE_unlock:  unlocking filehandle >$fh<... " if ($DEBUG);
	flock($fh, $LOCK_UN);
	close($fh);
	print STDERR "done\n" if ($DEBUG);
    }

    return(1);
}

#
# Cobalt::Util::FILE_unlock_rw
#
# inputs:  $filein = full path to file
#          $fhin = input filehandle returned by FILE_lock_rw
#          $fhout = output filehandle returned by FILE_lock_rw
# return:  1 if succes
#
sub FILE_unlock_rw
{
    my ($filein,$fhin,$fhout) = @_;
    my $LOCK_UN = 8;
    my $Lockdir = "/etc/locks";

    # extract filename from fully-qualified filename
    my @fary = split("/", $filein);
    my $ret = pop(@fary);
    my $fileout = "$Lockdir/".pop(@fary)."_$ret";

    print STDERR "FILE_unlock_rw:  unlocking $fileout... " if ($DEBUG);
    flock($fhout, $LOCK_UN);
    close($fhout);
    print STDERR "done\n" if ($DEBUG);

    print STDERR "FILE_unlock_rw:  moving $fileout to $filein... " if ($DEBUG);
    qx[ mv -f $fileout $filein ];
    print STDERR "done\n" if ($DEBUG);

    print STDERR "FILE_unlock_rw:  unlocking $filein... " if ($DEBUG);
    flock($fhin, $LOCK_UN);
    close($fhin);
    print STDERR "done\n" if ($DEBUG);

    return(1);
}

# 
# Cobalt::Util::lock_ui
# 
# inputs: 1 to lock, 0 to unlock, 2 to check lock
#
# return:  
#
sub lock_ui 
{
  my ($on) = @_;
  my $Uilockfile="/etc/locks/LCK..ui"; 
 
  if ( $on eq 1 ) {
    open( LOCKER, ">$Uilockfile" );
    close( LOCKER );
  } else {
    if ( $on eq 0 ) {
      unlink( "$Uilockfile" );
    } else {
      return ( -e $Uilockfile );
    }
  }

}


#
# Cobalt::Util::lock_services
#
# inputs: 1 to lock, 0 to unlock, 2 to check lock
#
# return:
#
sub lock_services
{
  my ($on) = @_;
  my $Servicelockfile="/etc/locks/LCK..services";

  if ( $on eq 1 ) {
    open( LOCKER, ">$Servicelockfile" );
    close( LOCKER );
  } else {
    if ( $on eq 0 ) {
      unlink( "$Servicelockfile" );
    } else {
      return ( -e $Servicelockfile );
    }
  }
}


# 
# Cobalt::Util::parse_cgipost
# 
# inputs: none explicitly, obtained from environment, both POST and GET
#
# return: hash of CGI variable name key and the variable definition as target
#
sub parse_cgipost
{
    my ($k,@equality,%fields);
    my $input = '';

    read(STDIN,$input,$ENV{"CONTENT_LENGTH"}) if ($ENV{"CONTENT_LENGTH"} > 0 );
    $input .= $ENV{"QUERY_STRING"}."&";

    foreach $_ ( split(/&/,$input) )
        {
        /(.*)=(.*)/;
        my $left = $1;
        my $right = $2;

        $left =~ s/\%(.{2})/sprintf("%c",hex($1))/eg;
        $left =~ tr/+/ /s;

        $right =~ s/\%(.{2})/sprintf("%c",hex($1))/eg;
        $right =~ tr/+/ /s;
        $right =~ s/[#\$\*\\\^\(\)\n]/ /og;
        $right =~ s/\+/ /g;
        $right =~ s/  / /g;
        $right =~ s/\"/\"\"/g;

        if ($fields{"$left"})
            {
            $fields{"$left"} .= ", $right" unless ($right eq "");
            }
        else
            {
            $fields{"$left"} = $right;
            }
        }

    $fields{"remote_addr"} = $ENV{"REMOTE_ADDR"};
    $fields{"remote_host"} = $ENV{"REMOTE_HOST"};

    return(%fields);
}

# Cobalt::Util::update_status($filehandle, $arrayref)
#
# writes the entries in the array referenced by $arrayref to $filehandle
# in a format easily understood and returned by Cobalt::Util::read_status
# $filehandle can be an open file handle or the full path to a file
# 
sub update_status
{
	my $fh = shift;
	my $status = shift;

	# for the call to fcntl, still not sure what it could be used for
        my $buff;
	my $close_file = 0;

	# check if $fh is an open filehandle or a filename
	if (not ref $fh) {
		$fh = new FileHandle(">$fh");
		if (not defined $fh) {
			return undef;
		}
		$close_file = 1;
	}

        fcntl $fh, F_WRLCK, $buff;

        seek $fh, 0, 0;

	for my $entry (@{ $status }) {
		print $fh "$entry\n";
	}

        fcntl $fh, F_UNLCK, $buff;

	$fh->close() if $close_file;
}

# Cobalt::Util::read_status($filehandle)
#
# essentially slurps in an entire file and returns an array with one line per
# entry, but with the added plus of doing file locking on the file handle
# $filehandle can be an open file handle or the full path to a file to read
#
sub read_status
{
	my $fh = shift;
	my $buff;
	my $close_file = 0;

	if (not ref $fh) {
		$fh = new FileHandle($fh);
		if (not defined $fh) {
			return undef;
		}
		$close_file = 1;
	}

        fcntl $fh, F_RDLCK, $buff;

        my @info = <$fh>;

        fcntl $fh, F_UNLCK, $buff;

	$fh->close() if $close_file;

	chomp @info;
	
	return @info;
}

# Cobalt::Util::untar_name($file_name, $desired_encoding)
#
# This probably only applies to Japanese.  This is used, because tar messes up 
# and turns double-byte characters
# into 6 byte long strings when doing 'tar tzf' on an archive which
# looks really ugly when displayed in the ui, and breaks selective restore.
# This fixes it by returning the 6 byte sequences to double-byte characters, and
# returns the new string encoded via JCode in the $desired_encoding (eg. euc,
# jis, sjis)
#
sub untar_name
{
	my $name = shift;
	my $encoding = shift;

	$name =~ s/\\(\d\d\d)/pack('C', oct($1))/ge;
	$name =~ s/\\\\/\\/g;
	
	# if no desired encoding was given just return the fixed string
	if (not $encoding) {
		return $name;
	} else {
		return Jcode::convert($name, $encoding);
	}
}	

1;

