#!/bin/sh
#########################################################################
#                                                                        #
#            Copyright (C) 1997-2000 Cobalt Networks, Inc                #
#                          All rights reserved                           #
#                                                                        #
# Filename: make_raid.sh                                                 #
#                                                                        #
# Author(s): Timothy Stonis <tim@cobalt.com>,                            #
#            Adrian Sun <asun@cobalt.com>                                #
#                                                                        #
# Description: Convert between RAID types                                #
#                                                                        #
##########################################################################

# DEBUG
# set -x
raid_tab="/etc/raidtab"
md4_entry="/tmp/raidtab.md4"
home_temp="/tmp/home.tar"
raid_status="/tmp/raidstatus"
target_level=$1
services="crond sendmail httpd postgresql snmpd cced.init quota" 

function toggle_services {

  for service in $services; do
    if [ -e /etc/rc.d/rc3.d/S*${service} ]; then
      /etc/rc.d/init.d/$service $1
    fi
  done

}

function check_and_tar {

  avail=$(df `dirname $1` | grep "^\/" | awk '{print $4}')
  need=$(du -s /home | awk '{print $1}')

  if [ $need -gt $avail ]; then
    echo "ERROR: Not enough disk space. Exiting"
    echo "ERROR: Not enough disk space. Exiting" > $raid_status
    exit 1
  fi

  # Tar up /home
  tar -cPf $1 /home

}

function clean_on_exit {

  rm $home_temp > /dev/null 2>&1
  rm $md4_entry > /dev/null 2>&1
  toggle_services start

}

function unmount_and_raid {

  umount  /dev/md4

  if [ $? -ne 0 ]; then
    echo "ERROR: Couldn't umount /home. Exiting."
    echo "ERROR: Couldn't umount /home." > $raid_status
    clean_on_exit
    exit 1
  fi

  # Now stop the raid array
  raidstop /dev/md4

  if [ $? -ne 0 ]; then
    echo "Problem stopping raid. Exiting."
    exit 1
  fi

  # Now make the new type of RAID
  cp $raid_tab /etc/raidtab.bak
  cp ${raid_tab}.new $raid_tab
  mkraid -q --really-force /dev/md4

  # Make a filesystem on it 
  # FIXME: Use a high performance chunk size
  mke2fs -m 0 /dev/md4

  # Mount the filesystem up
  mount /dev/md4 /home

  # Put our stuff back 
  (cd /; tar -xPf $home_temp)
 
  # Remove the tar file
  rm $home_temp 

  msg=`gettext --domain ui raidstep2`
  echo $msg > $raid_status

  /etc/rc.d/init.d/quota start

}

function new_raidtab {

  export current_level target_level
  cat $raid_tab | awk '
    BEGIN { current=ENVIRON["current_level"]
            target=ENVIRON["target_level"]
            hit=0 }
    /^raiddev.*md4.*/,/^device.*/ {
        hit=1 
        if ( $0 ~ /^raid-level/ ) {
          printf( "raid-level\t\t%s\n",target ) 
        } else {
          if ( $0 ~ /^chunk-size/ ) {
            if ( target == 5 ) {
              printf( "chunk-size\t\t64\n");
              printf( "parity-algorithm\tleft-symmetric\n");
            } else {
              printf( "chunk-size\t\t64\n");
            }
           } else {
             if ( $0 !~ /^parity/ ) {
               print
             }
           }
         } 
      }

   {
      if ( hit != 1 ) {
        print
      } 
      hit=0
    }

  ' > /tmp/raidtab.new

  mv /tmp/raidtab.new ${raid_tab}.new    
   

}

if [ X$target_level = X ]; then
  target_level=0
fi

msg=`gettext --domain ui raidstep1`
echo $msg > $raid_status

cat $raid_tab | sed -n -e "/^raiddev.*md4/,/^raiddev/ p" > $md4_entry
numdisks=$( cat $md4_entry | grep -c "^device.*dev.*" )
current_level=$( cat $md4_entry | grep "^raid-level" | awk '{print $2}')

echo "current is: $current_level"
echo "target is: $target_level"

if [ $current_level = $target_level ]; then
  echo "Same level. Exiting."
  echo "DONE: Same level, no change" > $raid_status
  clean_on_exit
  exit 0 
fi

# Handle going from 0 
if [ $current_level = 0 ]; then
  if [ $target_level -ne 5 -a $target_level -ne 1 ]; then
    echo "Can't handle transition to RAID $target_level. Exiting."
    exit 1
  fi

  if [ $target_level = 1 -a $numdisks != 2 ]; then
    echo "Can't transition to RAID 1 with $numdisks disks. Exiting."
    exit 1
  fi

  new_raidtab 
  toggle_services stop
  check_and_tar $home_temp 
  # For some reason, postgres won't die
  killall postmaster
  unmount_and_raid
  msg=`gettext --domain ui raidstep3`
  echo $msg > $raid_status
  clean_on_exit
  echo "DONE" > $raid_status
  exit 0

fi

# Handle going to 0 from 5 and 1 
if [ $target_level = 0 ]; then

  new_raidtab 
  toggle_services stop
  check_and_tar $home_temp 
  killall postmaster
  unmount_and_raid 
  msg=`gettext --domain ui raidstep3`
  echo $msg > $raid_status
  clean_on_exit
  echo "DONE" > $raid_status
  exit 0

else
  echo "Can't go to RAID $target_level from RAID $current_level. Exiting."
  exit 1
fi
